import argparse, pandas as pd


def joke():
    sdfds
    parser = argparse.ArgumentParser()

    parser.add_argument('-e', '--exp_group_list')
    parser.add_argument('-sb', '--savedir_base', required=True)
    parser.add_argument('-ei', '--exp_id', default=None)

    args = parser.parse_args()
    print(args)


def run_file():
    print('lol')